# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::ContainerRegistry::Mgmt::V2017_10_01
  #
  # Webhooks
  #
  class Webhooks
    include MsRestAzure

    #
    # Creates and initializes a new instance of the Webhooks class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [ContainerRegistryManagementClient] reference to the ContainerRegistryManagementClient
    attr_reader :client

    #
    # Gets the properties of the specified webhook.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Webhook] operation results.
    #
    def get(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      response = get_async(resource_group_name, registry_name, webhook_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets the properties of the specified webhook.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      get_async(resource_group_name, registry_name, webhook_name, custom_headers:custom_headers).value!
    end

    #
    # Gets the properties of the specified webhook.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MinLength': '1'" if !resource_group_name.nil? && resource_group_name.length < 1
      fail ArgumentError, 'registry_name is nil' if registry_name.nil?
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MaxLength': '50'" if !registry_name.nil? && registry_name.length > 50
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MinLength': '5'" if !registry_name.nil? && registry_name.length < 5
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !registry_name.nil? && registry_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?
      fail ArgumentError, 'webhook_name is nil' if webhook_name.nil?
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MaxLength': '50'" if !webhook_name.nil? && webhook_name.length > 50
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MinLength': '5'" if !webhook_name.nil? && webhook_name.length < 5
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !webhook_name.nil? && webhook_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/webhooks/{webhookName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'registryName' => registry_name,'webhookName' => webhook_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::Webhook.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates a webhook for a container registry with the specified parameters.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param webhook_create_parameters [WebhookCreateParameters] The parameters for
    # creating a webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Webhook] operation results.
    #
    def create(resource_group_name, registry_name, webhook_name, webhook_create_parameters, custom_headers:nil)
      response = create_async(resource_group_name, registry_name, webhook_name, webhook_create_parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param webhook_create_parameters [WebhookCreateParameters] The parameters for
    # creating a webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def create_async(resource_group_name, registry_name, webhook_name, webhook_create_parameters, custom_headers:nil)
      # Send request
      promise = begin_create_async(resource_group_name, registry_name, webhook_name, webhook_create_parameters, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
          result_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::Webhook.mapper()
          parsed_response = @client.deserialize(result_mapper, parsed_response)
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Deletes a webhook from a container registry.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    def delete(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      response = delete_async(resource_group_name, registry_name, webhook_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def delete_async(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      # Send request
      promise = begin_delete_async(resource_group_name, registry_name, webhook_name, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Updates a webhook with the specified parameters.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param webhook_update_parameters [WebhookUpdateParameters] The parameters for
    # updating a webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Webhook] operation results.
    #
    def update(resource_group_name, registry_name, webhook_name, webhook_update_parameters, custom_headers:nil)
      response = update_async(resource_group_name, registry_name, webhook_name, webhook_update_parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param webhook_update_parameters [WebhookUpdateParameters] The parameters for
    # updating a webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def update_async(resource_group_name, registry_name, webhook_name, webhook_update_parameters, custom_headers:nil)
      # Send request
      promise = begin_update_async(resource_group_name, registry_name, webhook_name, webhook_update_parameters, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
          result_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::Webhook.mapper()
          parsed_response = @client.deserialize(result_mapper, parsed_response)
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Lists all the webhooks for the specified container registry.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<Webhook>] operation results.
    #
    def list(resource_group_name, registry_name, custom_headers:nil)
      first_page = list_as_lazy(resource_group_name, registry_name, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Lists all the webhooks for the specified container registry.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_with_http_info(resource_group_name, registry_name, custom_headers:nil)
      list_async(resource_group_name, registry_name, custom_headers:custom_headers).value!
    end

    #
    # Lists all the webhooks for the specified container registry.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_async(resource_group_name, registry_name, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MinLength': '1'" if !resource_group_name.nil? && resource_group_name.length < 1
      fail ArgumentError, 'registry_name is nil' if registry_name.nil?
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MaxLength': '50'" if !registry_name.nil? && registry_name.length > 50
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MinLength': '5'" if !registry_name.nil? && registry_name.length < 5
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !registry_name.nil? && registry_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/webhooks'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'registryName' => registry_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::WebhookListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Triggers a ping event to be sent to the webhook.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [EventInfo] operation results.
    #
    def ping(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      response = ping_async(resource_group_name, registry_name, webhook_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Triggers a ping event to be sent to the webhook.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def ping_with_http_info(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      ping_async(resource_group_name, registry_name, webhook_name, custom_headers:custom_headers).value!
    end

    #
    # Triggers a ping event to be sent to the webhook.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def ping_async(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MinLength': '1'" if !resource_group_name.nil? && resource_group_name.length < 1
      fail ArgumentError, 'registry_name is nil' if registry_name.nil?
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MaxLength': '50'" if !registry_name.nil? && registry_name.length > 50
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MinLength': '5'" if !registry_name.nil? && registry_name.length < 5
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !registry_name.nil? && registry_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?
      fail ArgumentError, 'webhook_name is nil' if webhook_name.nil?
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MaxLength': '50'" if !webhook_name.nil? && webhook_name.length > 50
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MinLength': '5'" if !webhook_name.nil? && webhook_name.length < 5
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !webhook_name.nil? && webhook_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/webhooks/{webhookName}/ping'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'registryName' => registry_name,'webhookName' => webhook_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::EventInfo.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets the configuration of service URI and custom headers for the webhook.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [CallbackConfig] operation results.
    #
    def get_callback_config(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      response = get_callback_config_async(resource_group_name, registry_name, webhook_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets the configuration of service URI and custom headers for the webhook.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_callback_config_with_http_info(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      get_callback_config_async(resource_group_name, registry_name, webhook_name, custom_headers:custom_headers).value!
    end

    #
    # Gets the configuration of service URI and custom headers for the webhook.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_callback_config_async(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MinLength': '1'" if !resource_group_name.nil? && resource_group_name.length < 1
      fail ArgumentError, 'registry_name is nil' if registry_name.nil?
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MaxLength': '50'" if !registry_name.nil? && registry_name.length > 50
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MinLength': '5'" if !registry_name.nil? && registry_name.length < 5
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !registry_name.nil? && registry_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?
      fail ArgumentError, 'webhook_name is nil' if webhook_name.nil?
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MaxLength': '50'" if !webhook_name.nil? && webhook_name.length > 50
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MinLength': '5'" if !webhook_name.nil? && webhook_name.length < 5
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !webhook_name.nil? && webhook_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/webhooks/{webhookName}/getCallbackConfig'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'registryName' => registry_name,'webhookName' => webhook_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::CallbackConfig.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists recent events for the specified webhook.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<Event>] operation results.
    #
    def list_events(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      first_page = list_events_as_lazy(resource_group_name, registry_name, webhook_name, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Lists recent events for the specified webhook.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_events_with_http_info(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      list_events_async(resource_group_name, registry_name, webhook_name, custom_headers:custom_headers).value!
    end

    #
    # Lists recent events for the specified webhook.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_events_async(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MinLength': '1'" if !resource_group_name.nil? && resource_group_name.length < 1
      fail ArgumentError, 'registry_name is nil' if registry_name.nil?
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MaxLength': '50'" if !registry_name.nil? && registry_name.length > 50
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MinLength': '5'" if !registry_name.nil? && registry_name.length < 5
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !registry_name.nil? && registry_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?
      fail ArgumentError, 'webhook_name is nil' if webhook_name.nil?
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MaxLength': '50'" if !webhook_name.nil? && webhook_name.length > 50
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MinLength': '5'" if !webhook_name.nil? && webhook_name.length < 5
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !webhook_name.nil? && webhook_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/webhooks/{webhookName}/listEvents'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'registryName' => registry_name,'webhookName' => webhook_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::EventListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates a webhook for a container registry with the specified parameters.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param webhook_create_parameters [WebhookCreateParameters] The parameters for
    # creating a webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Webhook] operation results.
    #
    def begin_create(resource_group_name, registry_name, webhook_name, webhook_create_parameters, custom_headers:nil)
      response = begin_create_async(resource_group_name, registry_name, webhook_name, webhook_create_parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates a webhook for a container registry with the specified parameters.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param webhook_create_parameters [WebhookCreateParameters] The parameters for
    # creating a webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_create_with_http_info(resource_group_name, registry_name, webhook_name, webhook_create_parameters, custom_headers:nil)
      begin_create_async(resource_group_name, registry_name, webhook_name, webhook_create_parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates a webhook for a container registry with the specified parameters.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param webhook_create_parameters [WebhookCreateParameters] The parameters for
    # creating a webhook.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_create_async(resource_group_name, registry_name, webhook_name, webhook_create_parameters, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MinLength': '1'" if !resource_group_name.nil? && resource_group_name.length < 1
      fail ArgumentError, 'registry_name is nil' if registry_name.nil?
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MaxLength': '50'" if !registry_name.nil? && registry_name.length > 50
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MinLength': '5'" if !registry_name.nil? && registry_name.length < 5
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !registry_name.nil? && registry_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?
      fail ArgumentError, 'webhook_name is nil' if webhook_name.nil?
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MaxLength': '50'" if !webhook_name.nil? && webhook_name.length > 50
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MinLength': '5'" if !webhook_name.nil? && webhook_name.length < 5
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !webhook_name.nil? && webhook_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?
      fail ArgumentError, 'webhook_create_parameters is nil' if webhook_create_parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::WebhookCreateParameters.mapper()
      request_content = @client.serialize(request_mapper,  webhook_create_parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/webhooks/{webhookName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'registryName' => registry_name,'webhookName' => webhook_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:put, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 201
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::Webhook.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end
        # Deserialize Response
        if status_code == 201
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::Webhook.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Deletes a webhook from a container registry.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    #
    def begin_delete(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      response = begin_delete_async(resource_group_name, registry_name, webhook_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # Deletes a webhook from a container registry.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_delete_with_http_info(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      begin_delete_async(resource_group_name, registry_name, webhook_name, custom_headers:custom_headers).value!
    end

    #
    # Deletes a webhook from a container registry.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_delete_async(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MinLength': '1'" if !resource_group_name.nil? && resource_group_name.length < 1
      fail ArgumentError, 'registry_name is nil' if registry_name.nil?
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MaxLength': '50'" if !registry_name.nil? && registry_name.length > 50
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MinLength': '5'" if !registry_name.nil? && registry_name.length < 5
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !registry_name.nil? && registry_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?
      fail ArgumentError, 'webhook_name is nil' if webhook_name.nil?
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MaxLength': '50'" if !webhook_name.nil? && webhook_name.length > 50
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MinLength': '5'" if !webhook_name.nil? && webhook_name.length < 5
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !webhook_name.nil? && webhook_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/webhooks/{webhookName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'registryName' => registry_name,'webhookName' => webhook_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:delete, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202 || status_code == 204
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?

        result
      end

      promise.execute
    end

    #
    # Updates a webhook with the specified parameters.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param webhook_update_parameters [WebhookUpdateParameters] The parameters for
    # updating a webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Webhook] operation results.
    #
    def begin_update(resource_group_name, registry_name, webhook_name, webhook_update_parameters, custom_headers:nil)
      response = begin_update_async(resource_group_name, registry_name, webhook_name, webhook_update_parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Updates a webhook with the specified parameters.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param webhook_update_parameters [WebhookUpdateParameters] The parameters for
    # updating a webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_update_with_http_info(resource_group_name, registry_name, webhook_name, webhook_update_parameters, custom_headers:nil)
      begin_update_async(resource_group_name, registry_name, webhook_name, webhook_update_parameters, custom_headers:custom_headers).value!
    end

    #
    # Updates a webhook with the specified parameters.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param webhook_update_parameters [WebhookUpdateParameters] The parameters for
    # updating a webhook.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_update_async(resource_group_name, registry_name, webhook_name, webhook_update_parameters, custom_headers:nil)
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MinLength': '1'" if !resource_group_name.nil? && resource_group_name.length < 1
      fail ArgumentError, 'registry_name is nil' if registry_name.nil?
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MaxLength': '50'" if !registry_name.nil? && registry_name.length > 50
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'MinLength': '5'" if !registry_name.nil? && registry_name.length < 5
      fail ArgumentError, "'registry_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !registry_name.nil? && registry_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?
      fail ArgumentError, 'webhook_name is nil' if webhook_name.nil?
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MaxLength': '50'" if !webhook_name.nil? && webhook_name.length > 50
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'MinLength': '5'" if !webhook_name.nil? && webhook_name.length < 5
      fail ArgumentError, "'webhook_name' should satisfy the constraint - 'Pattern': '^[a-zA-Z0-9]*$'" if !webhook_name.nil? && webhook_name.match(Regexp.new('^^[a-zA-Z0-9]*$$')).nil?
      fail ArgumentError, 'webhook_update_parameters is nil' if webhook_update_parameters.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::WebhookUpdateParameters.mapper()
      request_content = @client.serialize(request_mapper,  webhook_update_parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ContainerRegistry/registries/{registryName}/webhooks/{webhookName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'registryName' => registry_name,'webhookName' => webhook_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:patch, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 201
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::Webhook.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end
        # Deserialize Response
        if status_code == 201
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::Webhook.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists all the webhooks for the specified container registry.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [WebhookListResult] operation results.
    #
    def list_next(next_page_link, custom_headers:nil)
      response = list_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Lists all the webhooks for the specified container registry.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_next_with_http_info(next_page_link, custom_headers:nil)
      list_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Lists all the webhooks for the specified container registry.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::WebhookListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists recent events for the specified webhook.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [EventListResult] operation results.
    #
    def list_events_next(next_page_link, custom_headers:nil)
      response = list_events_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Lists recent events for the specified webhook.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_events_next_with_http_info(next_page_link, custom_headers:nil)
      list_events_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Lists recent events for the specified webhook.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_events_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::ContainerRegistry::Mgmt::V2017_10_01::Models::EventListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Lists all the webhooks for the specified container registry.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [WebhookListResult] which provide lazy access to pages of the
    # response.
    #
    def list_as_lazy(resource_group_name, registry_name, custom_headers:nil)
      response = list_async(resource_group_name, registry_name, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # Lists recent events for the specified webhook.
    #
    # @param resource_group_name [String] The name of the resource group to which
    # the container registry belongs.
    # @param registry_name [String] The name of the container registry.
    # @param webhook_name [String] The name of the webhook.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [EventListResult] which provide lazy access to pages of the response.
    #
    def list_events_as_lazy(resource_group_name, registry_name, webhook_name, custom_headers:nil)
      response = list_events_async(resource_group_name, registry_name, webhook_name, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_events_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

  end
end
